import os
import pickle

import log


class Cache:

    PATH = os.path.join(".cache", "coveragespace")

    def __init__(self):
        self._data = {}

    def _store(self):
        directory = os.path.dirname(self.PATH)
        if not os.path.exists(directory):
            os.makedirs(directory)

        text = pickle.dumps(self._data)
        with open(self.PATH, "wb") as f:
            f.write(text)

    def _load(self):
        try:
            with open(self.PATH, "rb") as f:
                text = f.read()
        except IOError as e:
            log.debug("Unable to read cache: %s", e)
            return

        try:
            data = pickle.loads(text)
        except (TypeError, KeyError, IndexError) as e:
            log.debug("Unable to parse cache: %s", e)

        if isinstance(data, dict):
            self._data = data
        else:
            log.debug("Invalid cache value: %s", data)
            self._data = {}

    def set(self, key, value):
        slug = self._slugify(key)
        log.debug("Setting cache key: %s", slug)
        self._data[slug] = value
        log.debug("Cached value: %s", value)
        self._store()

    def get(self, key, default=None):
        self._load()
        slug = self._slugify(key)
        log.debug("Getting cache key: %s", slug)
        value = self._data.get(slug, default)
        log.debug("Cached value: %s", value)
        return value

    def clear(self):
        log.debug("Clearing cache")
        self._data = {}
        self._store()

    @staticmethod
    def _slugify(key):
        try:
            url, data = key
        except ValueError:
            return key
        else:
            return url, tuple(data.items())



# pylint: disable=missing-docstring,unused-variable,unused-argument,expression-not-assigned,singleton-comparison

from expecter import expect

from .. import environments


def describe_ci():
    def when_off_ci(monkeypatch):
        monkeypatch.delenv("APPVEYOR", raising=False)
        monkeypatch.delenv("CI", raising=False)
        monkeypatch.delenv("CONTINUOUS_INTEGRATION", raising=False)
        monkeypatch.delenv("TRAVIS", raising=False)
        monkeypatch.delenv("DISABLE_COVERAGE", raising=False)

        expect(environments.ci()) == False

    def when_on_ci(monkeypatch):
        monkeypatch.setenv("TRAVIS", "true")

        expect(environments.ci()) == True



from importlib.metadata import PackageNotFoundError, version

try:
    __version__ = version("coveragespace")
except PackageNotFoundError:  # pragma: no cover
    __version__ = "(local)"

CLI = "coveragespace"
API = "https://api.coverage.space"

VERSION = "{0} v{1}".format(CLI, __version__)

del PackageNotFoundError
del version



"""Unit tests configuration file."""

import log


def pytest_configure(config):
    """Disable verbose output when running tests."""
    log.init(debug=True)

    terminal = config.pluginmanager.getplugin("terminal")
    terminal.TerminalReporter.showfspath = False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['coveragespace', 'coveragespace.tests']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4',
 'coverage>=4.0',
 'docopt>=0.6',
 'minilog>=2.0',
 'requests>=2.28,<3.0']

entry_points = \
{'console_scripts': ['coveragespace = coveragespace.cli:main']}

setup_kwargs = {
    'name': 'coveragespace',
    'version': '6.0.2',
    'description': 'A place to track your code coverage metrics.',
    'long_description': '# Overview\n\nThe official command-line client for [The Coverage Space](http://coverage.space).\n\n[![Unix Build Status](https://img.shields.io/travis/jacebrowning/coverage-space-cli/main.svg?label=unix)](https://travis-ci.org/jacebrowning/coverage-space-cli)\n[![Windows Build Status](https://img.shields.io/appveyor/ci/jacebrowning/coverage-space-cli/main.svg?label=window)](https://ci.appveyor.com/project/jacebrowning/coverage-space-cli)\n[![Coverage Status](https://img.shields.io/coveralls/jacebrowning/coverage-space-cli/main.svg)](https://coveralls.io/r/jacebrowning/coverage-space-cli)\n[![Scrutinizer Code Quality](https://img.shields.io/scrutinizer/g/jacebrowning/coverage-space-cli.svg)](https://scrutinizer-ci.com/g/jacebrowning/coverage-space-cli/?branch=main)\n[![PyPI Version](https://img.shields.io/pypi/v/coveragespace.svg)](https://pypi.org/project/coveragespace)\n[![PyPI License](https://img.shields.io/pypi/l/coveragespace.svg)](https://pypi.org/project/coveragespace)\n\n# Setup\n\n## Requirements\n\n- Python 3.7+\n\n## Installation\n\nInstall this tool globally with [pipx](https://pipxproject.github.io/pipx/) (or pip):\n\n```text\n$ pipx install coveragespace\n```\nor add it to your [Poetry](https://python-poetry.org/docs/) project:\n\n```text\n$ poetry add coveragespace\n```\n\n# Usage\n\nTo update the value for a test coverage metric:\n\n```text\n$ coveragespace update <metric>\n```\n\nwhere `<metric>` is one of:\n\n- **unit**\n- **integration**\n- **overall**\n\nFor example, after running unit tests with code coverage enabled:\n\n```text\n$ coveragespace update unit\n```\n\nwhich will attempt to extract the current coverage data from your project directory and compare that with the last known value. The coverage value can also be manually specified:\n\n```text\n$ coveragespace update <metric> <value>\n```\n',
    'author': 'Jace Browning',
    'author_email': 'jacebrowning@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://coverage.space/client/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# pylint: disable=unused-variable,expression-not-assigned,singleton-comparison

from unittest.mock import Mock, patch

from expecter import expect

from .. import cli


def describe_call():
    @patch("coveragespace.cache.Cache.get", Mock())
    def it_handles_invalid_response():
        expect(cli.call("slug", "metric", 42)) == False

    @patch("coveragespace.cache.Cache.get", Mock(return_value=None))
    @patch("coveragespace.cache.Cache.set", Mock(return_value=None))
    @patch("time.sleep", Mock())
    @patch("requests.put")
    def it_retries_500s(requests_put):
        requests_put.return_value = Mock(status_code=500)

        cli.call("slug", "metric", 42)

        expect(requests_put.call_count) == 3
